﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading;
using System.Windows.Forms;

namespace MusicPlay
{
    //public partial class Main : Form
    public partial class Main : Form
    {
        public Main()
        {
            InitializeComponent();
        }

        Thread thread = null; //线程
        Thread threadPlay = null; //播放线程
        public delegate void UpdateListDown(ListViewItem item); //定义更新委托
        List<string> musicList =new List<string>(); //播放列表集合
        int playIndex = 0; //播放歌曲的索引
        string playName = ""; //播放项的歌名

        public delegate void UpdateLink(bool status);
        public void UpdateSearch(bool status) 
        {
            if (status)
            {
                this.Invoke(new ThreadStart(delegate
                    {
                        link_search.Text = "搜索(默认搜索酷狗无损音质)";
                        link_search.Enabled = true;
                    }));
            }
        }

        /// <summary>
        /// 委托更新列表方法
        /// </summary>
        /// <param name="item"></param>
        public void UpdateDown(ListViewItem item) 
        {
            this.Invoke(new ThreadStart(delegate 
                {
                    lv_MusicDown.Items.Add(item);
                }));
        }

        private void Main_Load(object sender, EventArgs e)
        {
            string fileName = "";
            if (File.Exists(AppDomain.CurrentDomain.BaseDirectory + "List.music")) 
            {
                fileName = AppDomain.CurrentDomain.BaseDirectory + "List.music";
            }

            if (fileName != null && fileName != "")
            {
                string[] music_List = File.ReadAllLines(fileName, Encoding.Default);
                if (music_List.Length > 0)
                {
                    for (int i = 0; i < music_List.Length; i++)
                    {
                        string tempUrl = music_List[i];
                        string[] tempInfo = tempUrl.Split('◆');
                        ListViewItem item = new ListViewItem((i + 1).ToString());
                        item.SubItems.Add(tempInfo[0]);
                        item.SubItems.Add(tempInfo[1]);

                        //判断是否重复
                        if (!musicList.Contains(tempInfo[1]))
                        {
                            //将地址添加到过虑集合中
                            musicList.Add(tempInfo[1]);

                            //添加到播放列表中
                            axPlayMp3.currentPlaylist = axPlayMp3.newPlaylist("酷狗列表", "");
                            axPlayMp3.currentPlaylist.appendItem(axPlayMp3.newMedia(tempInfo[1]));
                            axPlayMp3.Ctlcontrols.play(); //自动播放

                            playName = tempInfo[0]; //添加歌曲名
                            //添加一行
                            lv_MusicList.Items.Add(item);
                        }
                    }
                    groupBox1.Text = "播放列表" + "(" + lv_MusicList.Items.Count + ")"; //统计更新数量
                }
            }
        }


        /// <summary>
        /// 搜索下载音乐
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void link_search_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            lv_MusicDown.Items.Clear();
            link_search.Enabled = false;
            link_search.Text = "正在搜索中...";

            if (text_search.Text != "" && text_search.Text != null) 
            {
                TaskDown.update = new UpdateListDown(UpdateDown); //更新搜索列表
                TaskDown.upLink = new UpdateLink(UpdateSearch); //更新状态
                thread = new Thread(new ThreadStart(delegate 
                    {
                        TaskDown task = new TaskDown(text_search.Text,1);
                    }));
                thread.IsBackground = true;
                thread.Start();
            }
            else
            {
                MessageBox.Show("错误,请先输入曲名!","提示",MessageBoxButtons.OK,MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// 清空搜索列表
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tsl_clearDownList_Click(object sender, EventArgs e)
        {
            lv_MusicDown.Items.Clear();
        }

        /// <summary>
        /// 搜索区复制歌曲地址菜单
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tsl_copyMusicUrl_Click(object sender, EventArgs e)
        {
            if (lv_MusicDown.Items.Count > 0) 
            {
                if (lv_MusicDown.SelectedIndices != null && lv_MusicDown.SelectedIndices.Count > 0)
                {
                    string strText = lv_MusicDown.Items[lv_MusicDown.SelectedIndices[0]].SubItems[3].Text;
                    Clipboard.SetDataObject(strText);

                    notifyIcon_msg.Visible = true;
                    string info = string.Format("地址已复制【{0}】", strText);
                    notifyIcon_msg.ShowBalloonTip(1500, "提示", info, ToolTipIcon.Info);
                }
            }
        }

        /// <summary>
        /// 复制歌曲播放提示音
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void notifyIcon_msg_BalloonTipShown(object sender, EventArgs e)
        {
            string path = AppDomain.CurrentDomain.BaseDirectory + "\\TipShownMsg.wav"; //.wav音频文件路径
            if (File.Exists(path))
            {
                System.Media.SoundPlayer player = new System.Media.SoundPlayer(path);
                player.Play();//简单播放一遍
            }
        }

        /// <summary>
        /// 搜索区下载菜单
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tsl_downLoad_Click(object sender, EventArgs e)
        {
            if (lv_MusicDown.SelectedItems.Count > 0)
            {
                MusicDown mDown = new MusicDown();
                mDown.Owner = this;
                mDown.ShowDialog();
            }
            else
            {
                MessageBox.Show("请先选中一行歌曲!","提示",MessageBoxButtons.OK,MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// 搜索区添加音乐菜单
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tsl_addmusic_Click(object sender, EventArgs e)
        {
            if (lv_MusicDown.Items.Count > 0)
            {
                if (lv_MusicDown.SelectedIndices != null && lv_MusicDown.SelectedIndices.Count > 0)
                {
                    string item1Str = (lv_MusicList.Items.Count + 1).ToString(); //第一列文本
                    string music_name = lv_MusicDown.Items[lv_MusicDown.SelectedIndices[0]].SubItems[1].Text; //第二列文本
                    string music_playurl = lv_MusicDown.Items[lv_MusicDown.SelectedIndices[0]].SubItems[3].Text; //第三列文本

                    string music_url = music_playurl;
                    if (music_url.Contains("http")) 
                    {
                        music_url = music_url.Substring(music_url.LastIndexOf("/")).Replace("/", string.Empty);
                    }

                    //添加音乐之前判断是否存在
                    if (!musicList.Contains(music_url)) 
                    {
                        musicList.Add(music_url);  //将音乐地址添加到集合中,过虑重复

                        //添加一行
                        lv_MusicList.Items.Add(new ListViewItem(new string[] { item1Str, music_name, music_playurl }));
                    }
                    else
                    {
                        MessageBox.Show("添加的歌曲已经存在，无需重复添加！", "提示", MessageBoxButtons.OK, MessageBoxIcon.Question);
                    }
                }
                groupBox1.Text = "播放列表" + "(" + lv_MusicList.Items.Count + ")"; //统计下数量
            }
        }

        /// <summary>
        /// 删除选中项
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tsm_musicListDel_Click(object sender, EventArgs e)
        {
            foreach (ListViewItem item in lv_MusicList.Items)
            {
                if (lv_MusicList.SelectedItems.Contains(item))
                {
                    int indexDel = lv_MusicList.Items.IndexOf(lv_MusicList.FocusedItem);
                    if (lv_MusicList.SelectedItems.Count != 0)
                    {
                        lv_MusicList.Items.RemoveAt(indexDel);//删除
                        groupBox1.Text = "播放列表" + "(" + lv_MusicList.Items.Count + ")"; //统计更新数量
                    }
                }
            }
        }

        /// <summary>
        /// 向上移动歌曲
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tsm_musicListUp_Click(object sender, EventArgs e)
        {
            if (lv_MusicList.SelectedItems.Count > 0 && lv_MusicList.SelectedItems[0].Index != 0)
            {
                lv_MusicList.BeginUpdate();
                foreach (ListViewItem lvi in lv_MusicList.SelectedItems)
                {
                    ListViewItem item = lvi;
                    int index = lvi.Index;
                    lv_MusicList.Items.RemoveAt(index);
                    lv_MusicList.Items.Insert(index - 1, item);
                }
                lv_MusicList.EndUpdate();
            }
            lv_MusicList.Focus(); 
        }

        /// <summary>
        /// 向下移动歌曲
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tsm_musicListDown_Click(object sender, EventArgs e)
        {
            if (lv_MusicList.SelectedItems.Count > 0 && lv_MusicList.SelectedItems[lv_MusicList.SelectedItems.Count - 1].Index < (lv_MusicList.Items.Count - 1))
            {
                lv_MusicList.BeginUpdate();
                int count = lv_MusicList.SelectedItems.Count;
                foreach (ListViewItem lvi in lv_MusicList.SelectedItems)
                {
                    ListViewItem item = lvi;
                    int index = lvi.Index;
                    lv_MusicList.Items.RemoveAt(index);
                    lv_MusicList.Items.Insert(index + count, item);
                }
                lv_MusicList.EndUpdate();
            }
            lv_MusicList.Focus(); 
        }

        /// <summary>
        /// 清空歌曲列表
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tsm_musicListClear_Click(object sender, EventArgs e)
        {
            lv_MusicList.Items.Clear();
            groupBox1.Text = "播放列表" + "(0)"; //统计更新数量
        }

        /// <summary>
        /// 导出歌曲列表
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tsm_musicListImport_Click(object sender, EventArgs e)
        {
            if (lv_MusicList.Items.Count > 0) 
            {
                if (File.Exists(AppDomain.CurrentDomain.BaseDirectory + "\\List.music")) 
                {
                    File.Delete(AppDomain.CurrentDomain.BaseDirectory + "\\List.music");
                }

                for (int i = 0; i < lv_MusicList.Items.Count; i++)
                {
                    string musicName = lv_MusicList.Items[i].SubItems[1].Text;
                    string musicUrl = lv_MusicList.Items[i].SubItems[2].Text;
                    //◆
                    File.AppendAllText(AppDomain.CurrentDomain.BaseDirectory + "\\List.music", musicName + "◆" + musicUrl+"\r\n", Encoding.Default);
                }
                MessageBox.Show("导出歌曲列表完毕!","提示",MessageBoxButtons.OK,MessageBoxIcon.Information);
            }
        }

        /// <summary>
        /// 导入歌曲列表
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tsm_musicListInput_Click(object sender, EventArgs e)
        {
            OpenFileDialog open = new OpenFileDialog();
            open.Filter = "文本文档|*.music";
            open.ShowDialog();
            string fileName = open.FileName;
            if (fileName != null && fileName != "") 
            {
                string[] music_List = File.ReadAllLines(fileName, Encoding.Default);
                if (music_List.Length > 0) 
                {
                    for (int i = 0; i < music_List.Length; i++)
                    {
                        string tempUrl = music_List[i];
                        string[] tempInfo = tempUrl.Split('◆');
                        ListViewItem item = new ListViewItem((i+1).ToString());
                        item.SubItems.Add(tempInfo[0]);
                        item.SubItems.Add(tempInfo[1]);

                        //处理导入的url
                        string music_url = tempInfo[1];
                        if (music_url.Contains("http"))
                        {
                            music_url = music_url.Substring(music_url.LastIndexOf("/")).Replace("/", string.Empty);
                        }

                        //判断是否重复
                        if (!musicList.Contains(music_url))
                        {
                            //将地址添加到过虑集合中
                            musicList.Add(music_url);

                            //添加到播放列表中
                            axPlayMp3.currentPlaylist = axPlayMp3.newPlaylist("酷狗列表", "");
                            axPlayMp3.currentPlaylist.appendItem(axPlayMp3.newMedia(tempInfo[1]));
                            playName = tempInfo[0]; //添加歌曲名

                            //添加一行
                            lv_MusicList.Items.Add(item);
                        }
                    }
                    groupBox1.Text = "播放列表"+"("+lv_MusicList.Items.Count+")"; //统计更新数量
                }
            }
        }

        /// <summary>
        /// 添加歌曲
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tsm_musicListAdd_Click(object sender, EventArgs e)
        {
            MusicAdd mAdd = new MusicAdd();
            mAdd.Owner = this;
            mAdd.ShowDialog();
        }

        /// <summary>
        /// 试听音乐列表中的歌曲
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tsl_testMusic_Click(object sender, EventArgs e)
        {
            if (lv_MusicDown.SelectedIndices != null && lv_MusicDown.SelectedIndices.Count > 0)
            {
                string item1Str = (lv_MusicList.Items.Count + 1).ToString(); //第一列文本
                playName = lv_MusicDown.Items[lv_MusicDown.SelectedIndices[0]].SubItems[1].Text; //第二列文本
                string music_url = lv_MusicDown.Items[lv_MusicDown.SelectedIndices[0]].SubItems[3].Text; //第三列文本，播放地址

                axPlayMp3.URL = music_url;
                axPlayMp3.Ctlcontrols.play();
            }
        }

        /// <summary>
        /// 双击播放器控件，禁止全屏
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void axPlayMp3_MouseDownEvent(object sender, AxWMPLib._WMPOCXEvents_MouseDownEvent e)
        {
            if (axPlayMp3.fullScreen == true)
            {
                axPlayMp3.fullScreen = false;
            }
        }

        /// <summary>
        /// 双击列表播放音乐
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void lv_MusicDown_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            if (lv_MusicDown.SelectedIndices != null && lv_MusicDown.SelectedIndices.Count > 0)
            {
                playIndex = lv_MusicDown.SelectedItems[0].Index; //将歌曲的播放索引存起来
                playName = lv_MusicDown.Items[lv_MusicDown.SelectedIndices[0]].SubItems[1].Text; //第二列文本
                string music_url = lv_MusicDown.Items[lv_MusicDown.SelectedIndices[0]].SubItems[3].Text; //第三列文本，播放地址

                axPlayMp3.URL = music_url;
                axPlayMp3.Ctlcontrols.play();
            }
        }

        /// <summary>
        /// 音乐列表中，播放右键选中的音乐
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tsm_musicListPlay_Click(object sender, EventArgs e)
        {
            if (lv_MusicList.SelectedIndices != null && lv_MusicList.SelectedIndices.Count > 0)
            {
                playIndex = lv_MusicList.SelectedItems[0].Index; //将歌曲的播放索引存起来
                playName = lv_MusicList.Items[lv_MusicList.SelectedIndices[0]].SubItems[1].Text; //第2列文本，播放歌曲
                string music_url = lv_MusicList.Items[lv_MusicList.SelectedIndices[0]].SubItems[2].Text; //第三列文本，播放地址

                axPlayMp3.URL = music_url;
                axPlayMp3.Ctlcontrols.play();
            }
        }

        /// <summary>
        /// 播放状态
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void axPlayMp3_PlayStateChange(object sender, AxWMPLib._WMPOCXEvents_PlayStateChangeEvent e)
        {
            //音乐播放器 V01
            this.Invoke(new ThreadStart(delegate 
                {
                    Main.ActiveForm.Text = "音乐播放器 V01 【 " + playName + " 】";
                }));
            
            //当前播放器的播放状态发生改变的是偶，判断当前音乐播放器的播放状态是否到达了Ended， 如果是则进行下一曲
            if (axPlayMp3.playState == WMPLib.WMPPlayState.wmppsMediaEnded)
            {
                //先设置播放源
                if (playIndex == lv_MusicList.Items.Count) 
                {
                    playIndex = 0;
                }

                //赋值播放歌曲
                axPlayMp3.URL = lv_MusicList.Items[playIndex].SubItems[2].Text; //歌曲地址
                playName = lv_MusicList.Items[playIndex].SubItems[1].Text; //歌曲名
                axPlayMp3.Ctlcontrols.play(); //立即播放

                playIndex++; //自加1 就等于下一个
            }
            else if (axPlayMp3.playState == WMPLib.WMPPlayState.wmppsReady)     //播放器准备就绪后播放
            {
                //捕获异常 并忽略异常
                try
                {
                    axPlayMp3.Ctlcontrols.play();
                }
                catch (Exception)
                {
                }
            }
        }

        /// <summary>
        /// 双击播放列表，播放音乐
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void lv_MusicList_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            if (lv_MusicList.SelectedIndices != null && lv_MusicList.SelectedIndices.Count > 0)
            {
                playIndex = lv_MusicList.SelectedItems[0].Index; //将歌曲的播放索引存起来
                playName = lv_MusicList.Items[lv_MusicList.SelectedIndices[0]].SubItems[1].Text; //第二列文本
                string music_url = lv_MusicList.Items[lv_MusicList.SelectedIndices[0]].SubItems[2].Text; //第三列文本，播放地址
                
                axPlayMp3.URL = music_url;
                axPlayMp3.Ctlcontrols.play();
            }
        }

        /// <summary>
        /// 定时器控件
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void timer_scroll_Tick(object sender, EventArgs e)
        {
        }

        private void Main_FormClosed(object sender, FormClosedEventArgs e)
        {
            //消息框中需要显示哪些按钮，此处显示“确定”和“取消”
            MessageBoxButtons messButton = MessageBoxButtons.OKCancel;
            //"确定要退出吗？"是对话框的显示信息，"退出系统"是对话框的标题
            //默认情况下，如MessageBox.Show("确定要退出吗？")只显示一个“确定”按钮。
            DialogResult dr = MessageBox.Show("窗口关闭后，播放列表将会消失！是否现在保存", "退出提醒", messButton);
            if (dr == DialogResult.OK)//如果点击“确定”按钮
            {
                if (lv_MusicList.Items.Count > 0)
                {
                    if (File.Exists(AppDomain.CurrentDomain.BaseDirectory + "\\List.music"))
                    {
                        File.Delete(AppDomain.CurrentDomain.BaseDirectory + "\\List.music");
                    }

                    for (int i = 0; i < lv_MusicList.Items.Count; i++)
                    {
                        string musicName = lv_MusicList.Items[i].SubItems[1].Text;
                        string musicUrl = lv_MusicList.Items[i].SubItems[2].Text;
                        //◆
                        File.AppendAllText(AppDomain.CurrentDomain.BaseDirectory + "\\List.music", musicName + "◆" + musicUrl + "\r\n", Encoding.Default);
                    }
                    MessageBox.Show("歌曲列表保存完毕!", "提示", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
            }
            else//如果点击“取消”按钮
            {
                Application.Exit();
            }
        }
    }
    #region ListView 无闪烁处理
    //首先，自定义一个类ListViewNF，继承自 System.Windows.Forms.ListView
    class ListViewNF : System.Windows.Forms.ListView
    {
        public ListViewNF()
        {
            // 开启双缓冲
            this.SetStyle(ControlStyles.OptimizedDoubleBuffer | ControlStyles.AllPaintingInWmPaint, true);
            this.SetStyle(ControlStyles.EnableNotifyMessage, true);
        }
        protected override void OnNotifyMessage(Message m)
        {
            if (m.Msg != 0x14)
            {
                base.OnNotifyMessage(m);
            }
        }
    }
    #endregion
}
